(defun gnus-group-list-unread-or-cached (level &optional lowest)
  "List groups with unread or cached articles."
  (interactive "P")
  (when level
    (setq level (prefix-numeric-value level)))
  (when (or (not level) (>= level gnus-level-zombie))
    (gnus-cache-open))
  (gnus-group-prepare-flat-predicate 
   (or level gnus-level-subscribed)
   #'(lambda (info)
       (let* ((marks (gnus-info-marks info))
	      (group (gnus-info-group info))
	      (params (gnus-info-params info))
	      (unread (car (gnus-gethash group gnus-newsrc-hashtb))))
	 (or
	  (if (eq unread t)	; Unactivated?
	      gnus-group-list-inactive-groups ; We list unactivated
	    (> unread 0))	; We list groups with unread articles
	  (and gnus-list-groups-with-ticked-articles
	       (cdr (assq 'tick (gnus-info-marks info))))
					; And groups with tickeds
	  ;; Check for permanent visibility.
	  (and gnus-permanently-visible-groups
	       (string-match gnus-permanently-visible-groups
			     group))
	  (memq 'visible params)
	  (cdr (assq 'visible params))
	  (assq 'cache marks))))
   lowest
   #'(lambda (group)
       (or (gnus-gethash group 
			 gnus-cache-active-hashtb)
	   ;; Cache active file might use "." 
	   ;; instead of ":".
	   (gnus-gethash 
	    (mapconcat 'identity
		       (split-string group ":")
		       ".")
	    gnus-cache-active-hashtb))))
  (goto-char (point-min))
  (gnus-group-position-point))

